//using HandsEnumerations;
using System;
using UnityEngine;

[Serializable]
public class HandsPressure{

	//attributes marked as public for json capability
	//TODO: avoid it! find a way to JSON encode private attributes.
	/**
	 * Thumb (L) tip pressure status.
	 */
	public Pressure LThumbPressure;

	/**
	 * Index (L) tip pressure status.
	 */
	public Pressure LIndexPressure;

	/**
	 * Middle (L) tip pressure status.
	 */
	public Pressure LMiddlePressure;

	/**
	 * Ring (L) tip pressure status.
	 */
	public Pressure LRingPressure;

	/**
	 * Pinky (L) tip pressure status.
	 */
	public Pressure LPinkyPressure;

	/**
	 * Thumb (R) tip pressure status.
	 */
	public Pressure RThumbPressure;

	/**
	 * Index (R) tip pressure status.
	 */
	public Pressure RIndexPressure;

	/**
	 * Middle (R) tip pressure status.
	 */
	public Pressure RMiddlePressure;

	/**
	 * Ring (R) tip pressure status.
	 */
	public Pressure RRingPressure;

	/**
	 * Pinky (R) tip pressure status.
	 */
	public Pressure RPinkyPressure;

	public HandsPressure(){
		//init every fingertip to NO_PRESSURE
		this.LThumbPressure = Pressure.NO_PRESSURE;
		this.LIndexPressure = Pressure.NO_PRESSURE;
		this.LMiddlePressure = Pressure.NO_PRESSURE;
		this.LRingPressure = Pressure.NO_PRESSURE;
		this.LPinkyPressure = Pressure.NO_PRESSURE;
		this.RThumbPressure = Pressure.NO_PRESSURE;
		this.RIndexPressure = Pressure.NO_PRESSURE;
		this.RMiddlePressure = Pressure.NO_PRESSURE;
		this.RRingPressure = Pressure.NO_PRESSURE;
		this.RPinkyPressure = Pressure.NO_PRESSURE;
	}

	/**
	 * Set the given pressure in the given fingertip.
	 * @param finger Finger: the finger.
	 * @param pressure Pressure: the applied pressure.
	 */
	public void setPressure(Finger finger, Pressure pressure){
		//switch all fingers
		switch(finger){
			//case L_THUMB
			case Finger.L_THUMB:
				//set the given pressure
				LThumbPressure = pressure;
				//end the case
				break;
			//case L_INDEX
			case Finger.L_INDEX:
				//set the given pressure
				LIndexPressure = pressure;
				//end the case
				break;
			//case L_MIDDLE
			case Finger.L_MIDDLE:
				//set the given pressure
				LMiddlePressure = pressure;
				//end the case
				break;
			//case L_RING
			case Finger.L_RING:
				//set the given pressure
				LRingPressure = pressure;
				//end the case
				break;
			//case L_PINKY
			case Finger.L_PINKY:
				//set the given pressure
				LPinkyPressure = pressure;
				//end the case
				break;
			//case R_THUMB
			case Finger.R_THUMB:
				//set the given pressure
				RThumbPressure = pressure;
				//end the case
				break;
			//case R_INDEX
			case Finger.R_INDEX:
				//set the given pressure
				RIndexPressure = pressure;
				//end the case
				break;
			//case R_MIDDLE
			case Finger.R_MIDDLE:
				//set the given pressure
				RMiddlePressure = pressure;
				//end the case
				break;
			//case R_RING
			case Finger.R_RING:
				//set the given pressure
				RRingPressure = pressure;
				//end the case
				break;
			//case R_PINKY
			case Finger.R_PINKY:
				//set the given pressure
				RPinkyPressure = pressure;
				//end the case
				break;
		}
	}

	/**
	 * Serialize to JSON.
	 * @return String: JSON encapsulation of this object.
	 */
	public string toJSON(){
		//serialize and return
		return JsonUtility.ToJson(this);
	}
}